/*
 * ircd-ratbox: an advanced Internet Relay Chat Daemon(ircd). 
 * m_hackidle.c: Allows opers to hack their idle time.
 * 
 * Copyright (C) 2002 by the past and present ircd coders, and others.
 * 
 * m_idlehack.c - (C) Alan 'alz' Milford, 24th Nov 2003. (alz@efnet.net)
 * 
 * Usage Details:
 * 
 * This command is available for use by server operators (+o flag),
 * 
 * /addidle <seconds> - Adds <seconds> to your idle time 
 * /setlast <TS> - Sets last active time to <TS>
 * 
 * As idletimes are only held on the client's local server, this is not a
 * network-requirement.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place, Suite 330, Boston, MA  02111-1307 USA
 * 
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "channel_mode.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "whowas.h"
#include "irc_string.h"
#include "hash.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

/*
 * SND_HIDLE_WALLOPS
 * 
 * defining this will send a wallops message when an oper hacks their idle time.
 * 
 */
#define SND_HIDLE_WALLOPS

static void     mo_addidle(struct Client *, struct Client *, int, char **);
static void     mo_setlast(struct Client *, struct Client *, int, char **);

struct Message  addidle_msgtab = {
    "ADDIDLE", 0, 0, 1, 0, MFLG_SLOW, 0,
    {m_unregistered, m_not_oper, m_ignore, mo_addidle}
};
struct Message  setlast_msgtab = {
    "SETLAST", 0, 0, 1, 0, MFLG_SLOW, 0,
    {m_unregistered, m_not_oper, m_ignore, mo_setlast}
};

#ifndef STATIC_MODULES

void
_modinit(void)
{
    mod_add_cmd(&addidle_msgtab);
    mod_add_cmd(&setlast_msgtab);
}

void
_moddeinit(void)
{
    mod_del_cmd(&addidle_msgtab);
    mod_del_cmd(&setlast_msgtab);
}

const char     *_version = "1.8";
#endif


/*
 * mo_addidle 
 *      parv[0] = sender prefix 
 *      parv[1] = Number of seconds to add to idle time.
 */
static void 
mo_addidle(struct Client * client_p, struct Client * source_p,
       int parc, char *parv[])
{
    int             addidle = 0;
    int             newidle = 0;
    int             oldidle = 0;
    struct Client  *target_p;
    if (!IsOper(source_p)) {
        sendto_one(source_p, ":%s NOTICE %s :You are not opered.", me.name, parv[0]);
        return;
    }
    if ((target_p = find_person(parv[0])) == NULL) {
        return;
    }
    if (parc < 1) {
        sendto_one(source_p, ":%s NOTICE %s :No time specified", me.name, parv[0]);
        return;
    }
    addidle = atoi(parv[1]);
    newidle = (target_p->user->last - addidle);
    if (newidle < me.since) {
        sendto_one(source_p, ":%s NOTICE %s :[addidle] FAILED: %lu is before server start (%lu)", 
                    me.name, parv[0], newidle, me.since);
    } else {
        oldidle = target_p->user->last;
        if (newidle < target_p->firsttime) {
            target_p->firsttime = newidle;
            target_p->user->last = newidle;
            sendto_one(source_p, ":%s NOTICE %s :[addidle] set: %lu -> %lu - signon changed to match", 
                        me.name, parv[0], newidle, me.since);
            #ifdef SND_HIDLE_WALLOPS
            sendto_wallops_flags(UMODE_WALLOP, &me,
                         "[addidle] [%s!%s@%s] added %lu to their idletime (now %lu) - signon time changed to match",
                         source_p->name, source_p->username, source_p->host, addidle, target_p->user->last);
            #endif
        } else {
            target_p->user->last = newidle;
            sendto_one(source_p, ":%s NOTICE %s :[addidle] set: %lu -> %lu", me.name, parv[0], oldidle, newidle);
            #ifdef SND_HIDLE_WALLOPS
            sendto_wallops_flags(UMODE_WALLOP, &me,
                         "[addidle] [%s!%s@%s] added %lu to their idletime (now %lu)",
                         source_p->name, source_p->username, source_p->host, addidle, target_p->user->last);
            #endif
        }
    }
}

/*
 * mo_setlast 
 *      parv[0] = sender prefix 
 *      parv[1] = new TS of last active time.
 */
static void 
mo_setlast(struct Client * client_p, struct Client * source_p,
       int parc, char *parv[])
{
    int             newlast = 0;
    int             oldlast = 0;
    struct Client  *target_p;
    if (!IsOper(source_p)) {
        sendto_one(source_p, ":%s NOTICE %s :You are not teh opered.", me.name, parv[0]);
        return;
    }
    if ((target_p = find_person(parv[0])) == NULL) {
        return;
    }
    if (parc < 1) {
        sendto_one(source_p, ":%s NOTICE %s :No TS specified", me.name, parv[0]);
        return;
    }
    newlast = atoi(parv[1]);
    if (newlast < me.since) {
        sendto_one(source_p, ":%s NOTICE %s :[setlast] %lu is before server start (%lu)", me.name, parv[0], newlast, me.since);
    } else {
        oldlast = target_p->user->last;
        target_p->user->last = newlast;
        if (newlast < target_p->firsttime) {
            target_p->firsttime = newlast;
            sendto_one(source_p, ":%s NOTICE %s :[setlast] set: %lu -> %lu - signon changed to match", 
                        me.name, parv[0], oldlast, newlast);
        #ifdef SND_HIDLE_WALLOPS
            sendto_wallops_flags(UMODE_WALLOP, &me,
                         "[setlast] [%s!%s@%s] set: %lu -> %lu (signon changed to match)",
                         source_p->name, source_p->username, source_p->host, oldlast, newlast);
        #endif
        } else {
            sendto_one(source_p, ":%s NOTICE %s :[setlast] set: %lu -> %lu", me.name, parv[0], oldlast, newlast);
        #ifdef SND_HIDLE_WALLOPS
            sendto_wallops_flags(UMODE_WALLOP, &me,
                       "[setlast] [%s!%s@%s] set: %lu -> %lu",
                         source_p->name, source_p->username, source_p->host, oldlast, newlast);
        #endif
        }
    }
}

